/** @typedef {import('./FileSystemDirectoryHandle.js').default} FileSystemDirectoryHandle */

const native$2 = globalThis.showDirectoryPicker;

/**
 * @param {Object} [options]
 * @param {boolean} [options._preferPolyfill] If you rather want to use the polyfill instead of the native
 * @returns {Promise<FileSystemDirectoryHandle>}
 */
async function showDirectoryPicker (options = {}) {
  if (native$2 && !options._preferPolyfill) {
    return native$2(options)
  }

  const input = document.createElement('input');
  input.type = 'file';
  input.webkitdirectory = true;
  // Fallback to multiple files input for iOS Safari
  input.multiple = true;

  // See https://stackoverflow.com/questions/47664777/javascript-file-input-onchange-not-working-ios-safari-only
  input.style.position = 'fixed';
  input.style.top = '-100000px';
  input.style.left = '-100000px';
  document.body.appendChild(input);

  // Lazy load while the user is choosing the directory
  const p = Promise.resolve().then(function () { return util; });

  await new Promise(resolve => {
    input.addEventListener('change', resolve);
    input.click();
  });

  return p.then(mod => mod.getDirHandlesFromInput(input))
}

/** @typedef {import('./FileSystemFileHandle.js').default} FileSystemFileHandle */

const def = { accepts: [] };
const native$1 = globalThis.showOpenFilePicker;

/**
 * @param {Object} [options]
 * @param {boolean} [options.multiple] If you want to allow more than one file
 * @param {boolean} [options.excludeAcceptAllOption=false] Prevent user for selecting any
 * @param {Object[]} [options.accepts] Files you want to accept
 * @param {boolean} [options._preferPolyfill] If you rather want to use the polyfill instead of the native
 * @returns {Promise<FileSystemFileHandle[]>}
 */
async function showOpenFilePicker (options = {}) {
  const opts = { ...def, ...options };

  if (native$1 && !options._preferPolyfill) {
    return native$1(opts)
  }

  const input = document.createElement('input');
  input.type = 'file';
  input.multiple = opts.multiple;
  input.accept = (opts.accepts || [])
    .map(e => [
      ...(e.extensions || []).map(e => '.' + e),
      ...e.mimeTypes || []]
    )
    .flat()
    .join(',');

  // See https://stackoverflow.com/questions/47664777/javascript-file-input-onchange-not-working-ios-safari-only
  Object.assign(input.style, {
    position: 'fixed',
    top: '-100000px',
    left: '-100000px'
  });

  document.body.appendChild(input);

  // Lazy load while the user is choosing the directory
  const p = Promise.resolve().then(function () { return util; });

  await new Promise(resolve => {
    input.addEventListener('change', resolve, { once: true });
    input.click();
  });
  input.remove();

  return p.then(m => m.getFileHandlesFromInput(input))
}

/** @typedef {import('./FileSystemFileHandle.js').default} FileSystemFileHandle */

const native = globalThis.showSaveFilePicker;

/**
 * @param {Object} [options]
 * @param {boolean} [options.excludeAcceptAllOption=false] Prevent user for selecting any
 * @param {Object[]} [options.types] Files you want to accept
 * @param {string} [options.suggestedName] the name to fall back to when using polyfill
 * @param {string} [options._name] the name to fall back to when using polyfill
 * @param {boolean} [options._preferPolyfill] If you rather want to use the polyfill instead of the native
 * @return {Promise<FileSystemFileHandle>}
 */
async function showSaveFilePicker (options = {}) {
  if (native && !options._preferPolyfill) {
    return native(options)
  }

  if (options._name) {
    console.warn('deprecated _name, spec now have `suggestedName`');
    options.suggestedName = options._name;
  }

  const { FileSystemFileHandle } = await Promise.resolve().then(function () { return FileSystemFileHandle$1; });
  const { FileHandle } = await import('./downloader-DA2jrkXS.d.ts');
  return new FileSystemFileHandle(new FileHandle(options.suggestedName))
}

/** @typedef {import('./FileSystemDirectoryHandle.js').default} FileSystemDirectoryHandle */

if (globalThis.DataTransferItem && !DataTransferItem.prototype.getAsFileSystemHandle) {
  DataTransferItem.prototype.getAsFileSystemHandle = async function () {
    const entry = this.webkitGetAsEntry();
    const [
      { FileHandle, FolderHandle },
      { FileSystemDirectoryHandle },
      { FileSystemFileHandle }
    ] = await Promise.all([
      import('./sandbox-Bm_r0k_7.d.ts'),
      Promise.resolve().then(function () { return FileSystemDirectoryHandle$2; }),
      Promise.resolve().then(function () { return FileSystemFileHandle$1; })
    ]);

    return entry.isFile
      ? new FileSystemFileHandle(new FileHandle(entry, false))
      : new FileSystemDirectoryHandle(new FolderHandle(entry, false))
  };
}

/**
 * @param {object=} driver
 * @return {Promise<FileSystemDirectoryHandle>}
 */
async function getOriginPrivateDirectory (driver, options = {}) {
  if (!driver) {
    return globalThis.navigator?.storage?.getDirectory() || globalThis.getOriginPrivateDirectory()
  }
  const {FileSystemDirectoryHandle} = await Promise.resolve().then(function () { return FileSystemDirectoryHandle$2; });
  const module = await driver;
  const sandbox = await (module.default
    ? module.default(options)
    : module(options)
  );
  return new FileSystemDirectoryHandle(sandbox)
}

const config$1 = {
  ReadableStream: globalThis.ReadableStream,
  WritableStream: globalThis.WritableStream,
  TransformStream: globalThis.TransformStream,
  DOMException: globalThis.DOMException,
  Blob: globalThis.Blob,
  File: globalThis.File,
};

const { WritableStream } = config$1;

let FileSystemWritableFileStream$1 = class FileSystemWritableFileStream extends WritableStream {
  #writer
  constructor (writer) {
    super(writer);
    this.#writer = writer;
    // Stupid Safari hack to extend native classes
    // https://bugs.webkit.org/show_bug.cgi?id=226201
    Object.setPrototypeOf(this, FileSystemWritableFileStream.prototype);

    /** @private */
    this._closed = false;
  }

  async close () {
    this._closed = true;
    const w = this.getWriter();
    const p = w.close();
    w.releaseLock();
    return p
    // return super.close ? super.close() : this.getWriter().close()
  }

  /** @param {number} position */
  seek (position) {
    return this.write({ type: 'seek', position })
  }

  /** @param {number} size */
  truncate (size) {
    return this.write({ type: 'truncate', size })
  }

  // The write(data) method steps are:
  write (data) {
    if (this._closed) {
      return Promise.reject(new TypeError('Cannot write to a CLOSED writable stream'))
    }

    // 1. Let writer be the result of getting a writer for this.
    const writer = this.getWriter();

    // 2. Let result be the result of writing a chunk to writer given data.
    const result = writer.write(data);

    // 3. Release writer.
    writer.releaseLock();

    // 4. Return result.
    return result
  }
};

Object.defineProperty(FileSystemWritableFileStream$1.prototype, Symbol.toStringTag, {
  value: 'FileSystemWritableFileStream',
  writable: false,
  enumerable: false,
  configurable: true
});

Object.defineProperties(FileSystemWritableFileStream$1.prototype, {
  close: { enumerable: true },
  seek: { enumerable: true },
  truncate: { enumerable: true },
  write: { enumerable: true }
});

// Safari safari doesn't support writable streams yet.
if (
  globalThis.FileSystemFileHandle &&
  !globalThis.FileSystemFileHandle.prototype.createWritable &&
  !globalThis.FileSystemWritableFileStream
) {
  globalThis.FileSystemWritableFileStream = FileSystemWritableFileStream$1;
}

const kAdapter$2 = Symbol('adapter');

/**
 * @typedef {Object} FileSystemHandlePermissionDescriptor
 * @property {('read'|'readwrite')} [mode='read']
 */
class FileSystemHandle {
  /** @type {FileSystemHandle} */
  [kAdapter$2]

  /** @type {string} */
  name
  /** @type {('file'|'directory')} */
  kind

  /** @param {FileSystemHandle & {writable}} adapter */
  constructor (adapter) {
    this.kind = adapter.kind;
    this.name = adapter.name;
    this[kAdapter$2] = adapter;
  }

  /** @param {FileSystemHandlePermissionDescriptor} descriptor */
  async queryPermission (descriptor = {}) {
    const { mode = 'read' } = descriptor;
    const handle = this[kAdapter$2];

    if (handle.queryPermission) {
      return handle.queryPermission({mode})
    }

    if (mode === 'read') {
      return 'granted'
    } else if (mode === 'readwrite') {
      return handle.writable ? 'granted' : 'denied'
    } else {
      throw new TypeError(`Mode ${mode} must be 'read' or 'readwrite'`)
    }
  }

  async requestPermission ({mode = 'read'} = {}) {
    const handle = this[kAdapter$2];
    if (handle.requestPermission) {
      return handle.requestPermission({mode})
    }

    if (mode === 'read') {
      return 'granted'
    } else if (mode === 'readwrite') {
      return handle.writable ? 'granted' : 'denied'
    } else {
      throw new TypeError(`Mode ${mode} must be 'read' or 'readwrite'`)
    }
  }

  /**
   * Attempts to remove the entry represented by handle from the underlying file system.
   *
   * @param {object} options
   * @param {boolean} [options.recursive=false]
   */
  async remove (options = {}) {
    await this[kAdapter$2].remove(options);
  }

  /**
   * @param {FileSystemHandle} other
   */
  async isSameEntry (other) {
    if (this === other) return true
    if (
      (!other) ||
      (typeof other !== 'object') ||
      (this.kind !== other.kind) ||
      (!other[kAdapter$2])
    ) return false
    return this[kAdapter$2].isSameEntry(other[kAdapter$2])
  }
}

Object.defineProperty(FileSystemHandle.prototype, Symbol.toStringTag, {
  value: 'FileSystemHandle',
  writable: false,
  enumerable: false,
  configurable: true
});

// Safari safari doesn't support writable streams yet.
if (globalThis.FileSystemHandle) {
  globalThis.FileSystemHandle.prototype.queryPermission ??= function (descriptor) {
    return 'granted'
  };
}

const errors = {
  INVALID: ['seeking position failed.', 'InvalidStateError'],
  GONE: ['A requested file or directory could not be found at the time an operation was processed.', 'NotFoundError'],
  MISMATCH: ['The path supplied exists, but was not an entry of requested type.', 'TypeMismatchError'],
  MOD_ERR: ['The object can not be modified in this way.', 'InvalidModificationError'],
  SYNTAX: m => [`Failed to execute 'write' on 'UnderlyingSinkBase': Invalid params passed. ${m}`, 'SyntaxError'],
  SECURITY: ['It was determined that certain files are unsafe for access within a Web application, or that too many calls are being made on file resources.', 'SecurityError'],
  DISALLOWED: ['The request is not allowed by the user agent or the platform in the current context.', 'NotAllowedError']
};

const config = {
  writable: globalThis.WritableStream
};

async function fromDataTransfer (entries) {
  console.warn('deprecated fromDataTransfer - use `dt.items[0].getAsFileSystemHandle()` instead');
  const [memory, sandbox, fs] = await Promise.all([
    import('./memory-wsCBLkPS.d.ts'),
    import('./sandbox-Bm_r0k_7.d.ts'),
    Promise.resolve().then(function () { return FileSystemDirectoryHandle$2; })
  ]);

  const folder = new memory.FolderHandle('', false);
  folder._entries = entries.map(entry => entry.isFile
    ? new sandbox.FileHandle(entry, false)
    : new sandbox.FolderHandle(entry, false)
  );

  return new fs.FileSystemDirectoryHandle(folder)
}

async function getDirHandlesFromInput (input) {
  const { FolderHandle, FileHandle } = await import('./memory-wsCBLkPS.d.ts');
  const { FileSystemDirectoryHandle } = await Promise.resolve().then(function () { return FileSystemDirectoryHandle$2; });

  const files = Array.from(input.files);
  const rootName = files[0].webkitRelativePath.split('/', 1)[0];
  const root = new FolderHandle(rootName, false);

  files.forEach(file => {
    const path = file.webkitRelativePath.split('/');
    path.shift();
    const name = path.pop();

    const dir = path.reduce((dir, path) => {
      if (!dir._entries[path]) dir._entries[path] = new FolderHandle(path, false);
      return dir._entries[path]
    }, root);

    dir._entries[name] = new FileHandle(file.name, file, false);
  });

  return new FileSystemDirectoryHandle(root)
}

async function getFileHandlesFromInput (input) {
  const { FileHandle } = await import('./memory-wsCBLkPS.d.ts');
  const { FileSystemFileHandle } = await Promise.resolve().then(function () { return FileSystemFileHandle$1; });

  return Array.from(input.files).map(file =>
    new FileSystemFileHandle(new FileHandle(file.name, file, false))
  )
}

declare const util_config: typeof config;
declare const util_errors: typeof errors;
declare const util_fromDataTransfer: typeof fromDataTransfer;
declare const util_getDirHandlesFromInput: typeof getDirHandlesFromInput;
declare const util_getFileHandlesFromInput: typeof getFileHandlesFromInput;
declare namespace util {
  export { util_config as config, util_errors as errors, util_fromDataTransfer as fromDataTransfer, util_getDirHandlesFromInput as getDirHandlesFromInput, util_getFileHandlesFromInput as getFileHandlesFromInput };
}

const { GONE: GONE$1, MOD_ERR } = errors;

const kAdapter$1 = Symbol('adapter');

let FileSystemDirectoryHandle$1 = class FileSystemDirectoryHandle extends FileSystemHandle {
  /** @type {FileSystemDirectoryHandle} */
  [kAdapter$1]

  constructor (adapter) {
    super(adapter);
    this[kAdapter$1] = adapter;
  }

  /**
   * @param {string} name Name of the directory
   * @param {object} [options]
   * @param {boolean} [options.create] create the directory if don't exist
   * @returns {Promise<FileSystemDirectoryHandle>}
   */
  async getDirectoryHandle (name, options = {}) {
    if (name === '') {
      throw new TypeError(`Name can't be an empty string.`)
    }
    if (name === '.' || name === '..' || name.includes('/')) {
      throw new TypeError(`Name contains invalid characters.`)
    }
    options.create = !!options.create;
    const handle = await this[kAdapter$1].getDirectoryHandle(name, options);
    return new FileSystemDirectoryHandle(handle)
  }

  /** @returns {AsyncGenerator<[string, FileSystemHandle | FileSystemDirectoryHandle]>} */
  async * entries () {
    const {FileSystemFileHandle} = await Promise.resolve().then(function () { return FileSystemFileHandle$1; });

    for await (const [_, entry] of this[kAdapter$1].entries())
      yield [entry.name, entry.kind === 'file'
        ? new FileSystemFileHandle(entry)
        : new FileSystemDirectoryHandle(entry)];
  }

  /** @deprecated use .entries() instead */
  async * getEntries() {
    const {FileSystemFileHandle} = await Promise.resolve().then(function () { return FileSystemFileHandle$1; });
    console.warn('deprecated, use .entries() instead');
    for await (let entry of this[kAdapter$1].entries())
      yield entry.kind === 'file'
        ? new FileSystemFileHandle(entry)
        : new FileSystemDirectoryHandle(entry);
  }

  /**
   * @param {string} name Name of the file
   * @param {object} [options]
   * @param {boolean} [options.create] create the file if don't exist
   */
  async getFileHandle (name, options = {}) {
    const {FileSystemFileHandle} = await Promise.resolve().then(function () { return FileSystemFileHandle$1; });
    if (name === '') throw new TypeError(`Name can't be an empty string.`)
    if (name === '.' || name === '..' || name.includes('/')) {
      throw new TypeError(`Name contains invalid characters.`)
    }
    options.create = !!options.create;
    const handle = await this[kAdapter$1].getFileHandle(name, options);
    return new FileSystemFileHandle(handle)
  }

  /**
   * @param {string} name
   * @param {object} [options]
   * @param {boolean} [options.recursive]
   */
  async removeEntry (name, options = {}) {
    if (name === '') {
      throw new TypeError(`Name can't be an empty string.`)
    }
    if (name === '.' || name === '..' || name.includes('/')) {
      throw new TypeError(`Name contains invalid characters.`)
    }
    options.recursive = !!options.recursive; // cuz node's fs.rm require boolean
    return this[kAdapter$1].removeEntry(name, options)
  }

  async resolve (possibleDescendant) {
    if (await possibleDescendant.isSameEntry(this)) {
      return []
    }

    const openSet = [{ handle: this, path: [] }];

    while (openSet.length) {
      let { handle: current, path } = openSet.pop();

      for await (const entry of current.values()) {
        if (await entry.isSameEntry(possibleDescendant)) {
          return [...path, entry.name]
        }
        if (entry.kind === 'directory') {
          openSet.push({ handle: entry, path: [...path, entry.name] });
        }
      }
    }

    return null
  }

  async * keys () {
    for await (const [name] of this[kAdapter$1].entries())
      yield name;
  }

  async * values () {
    for await (const [_, entry] of this)
      yield entry;
  }

  [Symbol.asyncIterator]() {
    return this.entries()
  }
};

Object.defineProperty(FileSystemDirectoryHandle$1.prototype, Symbol.toStringTag, {
	value: 'FileSystemDirectoryHandle',
	writable: false,
	enumerable: false,
	configurable: true
});

Object.defineProperties(FileSystemDirectoryHandle$1.prototype, {
	getDirectoryHandle: { enumerable: true },
	entries: { enumerable: true },
	getFileHandle: { enumerable: true },
	removeEntry: { enumerable: true }
});

if (globalThis.FileSystemDirectoryHandle) {
  const proto = globalThis.FileSystemDirectoryHandle.prototype;

  proto.resolve = async function resolve (possibleDescendant) {
    if (await possibleDescendant.isSameEntry(this)) {
      return []
    }

    const openSet = [{ handle: this, path: [] }];

    while (openSet.length) {
      let { handle: current, path } = openSet.pop();

      for await (const entry of current.values()) {
        if (await entry.isSameEntry(possibleDescendant)) {
          return [...path, entry.name]
        }
        if (entry.kind === 'directory') {
          openSet.push({ handle: entry, path: [...path, entry.name] });
        }
      }
    }

    return null
  };

  // Safari allows us operate on deleted files,
  // so we need to check if they still exist.
  // Hope to remove this one day.
  async function ensureDoActuallyStillExist (handle) {
    const root = await navigator.storage.getDirectory();
    const path = await root.resolve(handle);
    if (path === null) { throw new DOMException(...GONE$1) }
  }

  const entries = proto.entries;
  proto.entries = async function * () {
    await ensureDoActuallyStillExist(this);
    yield * entries.call(this);
  };
  proto[Symbol.asyncIterator] = async function * () {
    yield * this.entries();
  };

  const removeEntry = proto.removeEntry;
  proto.removeEntry = async function (name, options = {}) {
    return removeEntry.call(this, name, options).catch(async err => {
      const unknown = err instanceof DOMException && err.name === 'UnknownError';
      if (unknown && !options.recursive) {
        const empty = (await entries.call(this).next()).done;
        if (!empty) { throw new DOMException(...MOD_ERR) }
      }
      throw err
    })
  };
}

declare namespace FileSystemDirectoryHandle$2 {
  export { FileSystemDirectoryHandle$1 as FileSystemDirectoryHandle, FileSystemDirectoryHandle$1 as default };
}

const { INVALID, SYNTAX, GONE } = errors;

// Safari doesn't support async createWritable streams yet.
if (
  globalThis.FileSystemFileHandle &&
  !globalThis.FileSystemFileHandle.prototype.createWritable
) {
  const wm = new WeakMap();

  let workerUrl;

  // Worker code that should be inlined (can't use any external functions)
  const code = () => {
    let fileHandle, handle;

    onmessage = async evt => {
      const port = evt.ports[0];
      const cmd = evt.data;
      switch (cmd.type) {
        case 'open':
          const file = cmd.name;

          let dir = await navigator.storage.getDirectory();

          for (const folder of cmd.path) {
            dir = await dir.getDirectoryHandle(folder);
          }

          fileHandle = await dir.getFileHandle(file);
          handle = await fileHandle.createSyncAccessHandle();
          break
        case 'write':
          handle.write(cmd.data, { at: cmd.position });
          handle.flush();
          break
        case 'truncate':
          handle.truncate(cmd.size);
          break
        case 'abort':
        case 'close':
          handle.close();
          break
      }

      port.postMessage(0);
    };
  };


  globalThis.FileSystemFileHandle.prototype.createWritable = async function (options) {
    // Safari only support writing data in a worker with sync access handle.
    if (!workerUrl) {
      const stringCode = `(${code.toString()})()`;
      const blob = new Blob([stringCode], {
        type: 'text/javascript'
      });
      workerUrl = URL.createObjectURL(blob);
    }
    const worker = new Worker(workerUrl, { type: 'module' });

    let position = 0;
    const textEncoder = new TextEncoder();
    let size = await this.getFile().then(file => file.size);

    const send = message => new Promise((resolve, reject) => {
      const mc = new MessageChannel();
      mc.port1.onmessage = evt => {
        if (evt.data instanceof Error) reject(evt.data);
        else resolve(evt.data);
        mc.port1.close();
        mc.port2.close();
        mc.port1.onmessage = null;
      };
      worker.postMessage(message, [mc.port2]);
    });

    // Safari also don't support transferable file system handles.
    // So we need to pass the path to the worker. This is a bit hacky and ugly.
    const root = await navigator.storage.getDirectory();
    const parent = await wm.get(this);
    const path = await root.resolve(parent);

    // Should likely never happen, but just in case...
    if (path === null) throw new DOMException(...GONE)
    await send({ type: 'open', path, name: this.name });

    if (options?.keepExistingData === false) {
      await send({ type: 'truncate', size: 0 });
      size = 0;
    }

    const ws = new FileSystemWritableFileStream({
      start: ctrl => {
      },
      async write(chunk) {
        const isPlainObject = chunk?.constructor === Object;

        if (isPlainObject) {
          chunk = { ...chunk };
        } else {
          chunk = { type: 'write', data: chunk, position };
        }

        if (chunk.type === 'write') {
          if (!('data' in chunk)) {
            await send({ type: 'close' });
            throw new DOMException(...SYNTAX('write requires a data argument'))
          }

          chunk.position ??= position;

          if (typeof chunk.data === 'string') {
            chunk.data = textEncoder.encode(chunk.data);
          }

          else if (chunk.data instanceof ArrayBuffer) {
            chunk.data = new Uint8Array(chunk.data);
          }

          else if (!(chunk.data instanceof Uint8Array) && ArrayBuffer.isView(chunk.data)) {
            chunk.data = new Uint8Array(chunk.data.buffer, chunk.data.byteOffset, chunk.data.byteLength);
          }

          else if (!(chunk.data instanceof Uint8Array)) {
            const ab = await new Response(chunk.data).arrayBuffer();
            chunk.data = new Uint8Array(ab);
          }

          if (Number.isInteger(chunk.position) && chunk.position >= 0) {
            position = chunk.position;
          }
          position += chunk.data.byteLength;
          size += chunk.data.byteLength;
        } else if (chunk.type === 'seek') {
          if (Number.isInteger(chunk.position) && chunk.position >= 0) {
            if (size < chunk.position) {
              throw new DOMException(...INVALID)
            }
            console.log('seeking', chunk);
            position = chunk.position;
            return // Don't need to enqueue seek...
          } else {
            await send({ type: 'close' });
            throw new DOMException(...SYNTAX('seek requires a position argument'))
          }
        } else if (chunk.type === 'truncate') {
          if (Number.isInteger(chunk.size) && chunk.size >= 0) {
            size = chunk.size;
            if (position > size) { position = size; }
          } else {
            await send({ type: 'close' });
            throw new DOMException(...SYNTAX('truncate requires a size argument'))
          }
        }

        await send(chunk);
      },
      async close () {
        await send({ type: 'close' });
        worker.terminate();
      },
      async abort (reason) {
        await send({ type: 'abort', reason });
        worker.terminate();
      },
    });

    return ws
  };

  const orig = FileSystemDirectoryHandle.prototype.getFileHandle;
  FileSystemDirectoryHandle.prototype.getFileHandle = async function (...args) {
    const handle = await orig.call(this, ...args);
    wm.set(handle, this);
    return handle
  };
}

const kAdapter = Symbol('adapter');

class FileSystemFileHandle extends FileSystemHandle {
  /** @type {FileSystemFileHandle} */
  [kAdapter]

  constructor (adapter) {
    super(adapter);
    this[kAdapter] = adapter;
  }

  /**
   * @param  {Object} [options={}]
   * @param  {boolean} [options.keepExistingData]
   * @returns {Promise<FileSystemWritableFileStream>}
   */
  async createWritable (options = {}) {
    return new FileSystemWritableFileStream$1(
      await this[kAdapter].createWritable(options)
    )
  }

  /**
   * @returns {Promise<File>}
   */
  async getFile () {
    return this[kAdapter].getFile()
  }
}

Object.defineProperty(FileSystemFileHandle.prototype, Symbol.toStringTag, {
  value: 'FileSystemFileHandle',
  writable: false,
  enumerable: false,
  configurable: true
});

Object.defineProperties(FileSystemFileHandle.prototype, {
  createWritable: { enumerable: true },
  getFile: { enumerable: true }
});

type FileSystemFileHandle$1_FileSystemFileHandle = FileSystemFileHandle;
declare const FileSystemFileHandle$1_FileSystemFileHandle: typeof FileSystemFileHandle;
declare namespace FileSystemFileHandle$1 {
  export { FileSystemFileHandle$1_FileSystemFileHandle as FileSystemFileHandle, FileSystemFileHandle as default };
}

export { FileSystemDirectoryHandle$1 as F, FileSystemFileHandle as a, FileSystemHandle as b, config$1 as c, FileSystemWritableFileStream$1 as d, errors as e, showOpenFilePicker as f, getOriginPrivateDirectory as g, showSaveFilePicker as h, showDirectoryPicker as s };
